#========================================================================================
#   Test offline - functions of GeoData.pm
#========================================================================================

use strict;
use Test::More tests => 14;
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Address::API::GeoData qw ( BuildAddress GetGeoLocation FilterAll FilterLocation );
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );


use Test::Mock::LWP::Dispatch;
use HTTP::Response;
use HTTP::Headers;
use HTTP::Status qw ( :constants :is status_message );
use HTTP::Request::Common qw (GET);
my $URI_JENA = 'http://maps.googleapis.com/maps/api/geocode/json?address=Leutragraben+1+07743+Jena+Germany&sensor=false';

use JSON qw (from_json);

sub GetJsonOk {
    my $json = <<'RAW';
{
   "results" : [
      {
         "address_components" : [
            {
               "long_name" : "1",
               "short_name" : "1",
               "types" : [ "street_number" ]
            },
            {
               "long_name" : "Leutragraben",
               "short_name" : "Leutragraben",
               "types" : [ "route" ]
            },
            {
               "long_name" : "Jena",
               "short_name" : "Jena",
               "types" : [ "locality", "political" ]
            },
            {
               "long_name" : "Jena",
               "short_name" : "Jena",
               "types" : [ "administrative_area_level_3", "political" ]
            },
            {
               "long_name" : "Thuringen",
               "short_name" : "TH",
               "types" : [ "administrative_area_level_1", "political" ]
            },
            {
               "long_name" : "Germany",
               "short_name" : "DE",
               "types" : [ "country", "political" ]
            },
            {
               "long_name" : "07743",
               "short_name" : "07743",
               "types" : [ "postal_code" ]
            }
         ],
         "formatted_address" : "Leutragraben 1, 07743 Jena, Germany",
         "geometry" : {
            "location" : {
               "lat" : 50.928870,
               "lng" : 11.584510
            },
            "location_type" : "ROOFTOP",
            "viewport" : {
               "northeast" : {
                  "lat" : 50.93021898029150,
                  "lng" : 11.58585898029150
               },
               "southwest" : {
                  "lat" : 50.92752101970850,
                  "lng" : 11.58316101970850
               }
            }
         },
         "partial_match" : true,
         "types" : [ "street_address" ]
      }
   ],
   "status" : "OK"
}
RAW
    return $json;
}

sub GetJsonZero {
    return <<'RAW';
{
   "results" : [],
   "status" : "ZERO_RESULTS"
}
RAW

}

sub GetLngLatJena {
    return {
          'lat' => '50.92887',
          'lng' => '11.58451'
        };
}

sub GetAddrJena {
    return  {
        Street  => 'Leutragraben 1',
        Zipcode => '07743',
        City => 'Jena',
        Country => 'Germany',
    };
}

sub TestFilterAll {
    my $hAll = FilterAll (HTTP::Response->new(HTTP_OK, status_message(HTTP_OK), [], GetJsonOk()) );
    is_deeply($hAll, from_json( GetJsonOk() ), 'FilterAll works' )
}

sub TestFilterLocation {
    my $hLocation = FilterLocation (HTTP::Response->new(HTTP_OK, status_message(HTTP_OK), [], GetJsonOk()));
    is_deeply ($hLocation, GetLngLatJena, 'FilterLocation works');
}

sub TestGetGeoLocation {
    local * DE_EPAGES::Address::API::GeoData::_GetUserAgent = \&MockGetUserAgent; ## no critic (Subroutines::ProtectPrivateSubs)
    my $hLocation = GetGeoLocation(GetAddrJena());
    is_deeply ($hLocation, GetLngLatJena, 'GetGeoLocation works');
}

sub TestGetGeoLocationError {
    local * DE_EPAGES::Address::API::GeoData::_GetUserAgent = \&MockGetUserAgent; ## no critic (Subroutines::ProtectPrivateSubs)
    $MAIN::JSON = '{"status" : "haha"}';
    my ($hResult, $Error) = (undef, undef);
    eval { $hResult = GetGeoLocation(GetAddrJena()); };
    $Error = GetError() if ExistsError();
    ok ($Error->message() =~ m/haha/, 'Error \'OTHER - status');
    ok ( $Error->message()=~ m/\Q$URI_JENA\E/ , 'Error \'OTHER - request URI');

    $MAIN::JSON = '{"status" : "OVER_QUERY_LIMIT"}';
    eval { $hResult = GetGeoLocation(GetAddrJena()); };
    $Error = GetError() if ExistsError();
    ok ($Error->message() =~ m/OVER_QUERY_LIMIT/, 'Error \'OVER_QUERY_LIMIT\' - status');
    ok ($Error->message() =~ m/\Q$URI_JENA\E/ , 'Error \'OVER_QUERY_LIMIT\' - request URI');

    $MAIN::JSON = '{"status" : "REQUEST_DENIED"}';
    eval { $hResult = GetGeoLocation(GetAddrJena()); };
    $Error = GetError() if ExistsError();
    ok ($Error->message() =~ m/REQUEST_DENIED/, 'Error \'REQUEST_DENIED\' - status');
    ok ($Error->message() =~ m/\Q$URI_JENA\E/ , 'Error \'REQUEST_DENIED\' - request URI');

    $MAIN::JSON = '{"status" : "INVALID_REQUEST"}';
    eval { $hResult = GetGeoLocation(GetAddrJena()); };
    $Error = GetError() if ExistsError();
    ok ($Error->message() =~ m/INVALID_REQUEST/, 'Error \'INVALID_REQUEST\' - status');
    ok ($Error->message() =~ m/\Q$URI_JENA\E/ , 'Error \'INVALID_REQUEST\' - request URI');

    $MAIN::JSON = GetJsonOk;
    eval { $hResult = GetGeoLocation(GetAddrJena()); };
    ok (! ExistsError, 'Status \'OK\' - status');

    $MAIN::JSON = GetJsonZero() ;
    eval { $hResult = GetGeoLocation(GetAddrJena()); };
    ok (! ExistsError, 'Status \'ZERO_RESULTS\'');
}

sub MockGetUserAgent {
    my $UA = LWP::UserAgent->new;
    my $Header = HTTP::Headers->new( 'Content-Type' => 'application/json; charset=UTF-8', 'Conection' => 'close' );
    my $Response = HTTP::Response->new( HTTP_OK, status_message(HTTP_OK), $Header, $MAIN::JSON ) ;
    my $Request = GET( $URI_JENA );
    $Response->request($Request);
    $UA->map( $URI_JENA,  $Response);

    return $UA;
}

sub TestBuildParameter {
    my $hAddr = {
        Street  => 'Diagonal, 435 5-1',
        City => '08036 Barcelona',
        Country => 'Spain',
    };
    my $Quoted = BuildAddress($hAddr);
    is( $Quoted, 'address=Diagonal%2C+435+5%C2%BA-1%C2%AA++08036+Barcelona+Spain', 'quote unicode input');
}

#========================================================================================
#   MAIN
#========================================================================================

RunScript ( Sub => sub {
        $MAIN::JSON = GetJsonOk();
        TestFilterAll();
        TestFilterLocation();
        TestGetGeoLocation();
        TestGetGeoLocationError();
        TestBuildParameter();
} );





