use strict;

use Test::Simple tests => 20;
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetCurrentDBHandle GetTestSite RunOnStore );

sub Main {
    my $dbi = GetCurrentDBHandle();
    my $driverName = $dbi->driverName;

    SKIP: {
        skip( 'dropIndexesIfExist is supported for mysql', 8 ) if $driverName ne 'mysql';

        ok( Init($dbi), 'Init' );

        TestIndices($dbi);
        TestForeignKeys($dbi);

        ok( CleanUp($dbi), 'CleanUp' );
    }

    return;
}

# EPG-24188, EPG-25936
sub TestIndices {
    my ($dbi) = @_;

    my $Index;
    my @aColumns = qw( displayname street zipcode city );
    foreach my $Column ( @aColumns ) {
        ok( $dbi->existsTableColumn( 'testaddress', $Column ), "column '$Column' exists" );
        $Index = 'i_' . $Column;
        $dbi->createIndex( 'testaddress', $Index, undef, [ $Column ] );
        ok( $dbi->existsIndex( 'testaddress', $Index), "Create index $Index" );
    }

    my $aIndexes = [ 'i_displayname', 'i_zipcode' ];
    $dbi->dropIndexesIfExist( 'testaddress', $aIndexes );
    foreach my $Index ( @{ $aIndexes } ) {
        ok( !$dbi->existsIndex( 'testaddress', $Index ), "Index $Index already dropped" );
    }
    ok( $dbi->existsIndex( 'testaddress', 'i_street' ), "Index $Index exists" );
    ok( $dbi->existsIndex( 'testaddress', 'i_city' ), "Index $Index exists" );

    $aIndexes = [ 'i_city' ];
    $dbi->dropIndexesIfExist( 'testaddress', $aIndexes );
    ok( !$dbi->existsIndex( 'testaddress', 'i_city' ), "Index $Index already dropped" );
    ok( $dbi->existsIndex( 'testaddress', 'i_street' ), "Index $Index exists" );

    return;
}

# EPG-28659
sub TestForeignKeys {
    my ($dbi) = @_;

    ok( $dbi->existsForeignKey('fk_testuser_addressid'),
        'foreign key \'fk_testuser_addressid\' exists' );
    ok( $dbi->existsForeignKeyByName('testuser', 'fk_testuser_addressid'),
        'foreign key \'fk_testuser_addressid\' exists by name' );
    ok( !$dbi->existsForeignKey('fk_testuser_parentid'),
        'foreign key could not be found');
    ok( $dbi->existsAnyForeignKeyOn('testuser', [ 'addressid' ], 'testaddress', [ 'addressid' ]),
        'existsAnyForeignKeyOn: foreign key \'addressid\' exists (1)' );

    return;
}

sub CleanUp {
    my ($dbi) = @_;

    $dbi->dropTableIfExists('testuser');
    $dbi->dropTableIfExists('testaddress');

    return 1;
}

sub Init {
    my ($dbi) = @_;

    CleanUp($dbi);
    $dbi->do(<<END_SQL);
CREATE TABLE testaddress (
    addressid INTEGER PRIMARY KEY,
    displayname VARCHAR(50) NULL,
    street VARCHAR(255) NULL,
    zipcode VARCHAR(25) NULL,
    city VARCHAR(60) NULL
)
END_SQL

    $dbi->do(<<END_SQL);
CREATE TABLE testuser (
    userid INTEGER PRIMARY KEY,
    name VARCHAR(50) NULL,
    addressid INTEGER DEFAULT NULL,
    parentid INTEGER DEFAULT NULL,
    KEY `i_testuser_addressid` (`addressid`),
    KEY `i_testusr_userid` (`userid`),
    CONSTRAINT `fk_testuser_addressid` FOREIGN KEY (`addressid`) REFERENCES `testaddress` (`addressid`),
    CONSTRAINT `fk_testusr_parentid` FOREIGN KEY (`parentid`) REFERENCES `testuser` (`userid`)
)
END_SQL

    return 1;
}

RunScript(
    Message => 'long',
    Sub     => sub {
        RunOnStore(
            Store => GetTestSite(),
            Sub   => \&Main,
        );
    }
);

1;